@@ -4,7 +4,12 @@ import matplotlib
 import matplotlib.pyplot as plt
 matplotlib.use("TkAgg")
 
-from variable_row_array import VarRowArray
+from pyJJAsim.variable_row_array import VarRowArray
+
+
+"""
+Embedded Graph Module
+"""
 
 class NotSingleComponentError(Exception):
     pass
@@ -33,38 +38,52 @@ class EmbeddedGraph:
     Requirements:
      - No self-loops
      - Simple (no multigraph)
-    Input:
-    x, y            (N,)
-    node1, node2    (E,) in range(N)
-
-    Methods:
-    get_faces()
-    is_planar()
-    get_areas()
-    get_centroids()
-    get_num_components()
-    get_boundary_faces()
 
     """
 
     def __init__(self, x, y, node1, node2, require_single_component=False,
                  require_planar_embedding=False, _edges_are_sorted=False):
+        """
+        Parameters
+        ----------
+        x, y: (N,) array
+            coordinates of nodes of embedded graph
+        node1, node2: (E,) int array in range(N)
+            endpoint nodes of edges in embedded graph. Nodes are referred to by their index in
+            the coordinate arrays.
+        require_single_component=False:
+            If True, an error is raised if the graph is not single-component
+        require_planar_embedding=False:
+            If True, an error is raised if the graph is not a planar embedding
+
+        Raises
+        ------
+        NonSimpleError
+            If graph is not simple
+        SelfLoopError
+            if graph contains self-loops
+        NotSingleComponentError
+            if graph has disconnected components and require_single_component=True
+        NotPlanarEmbeddingError
+            If graph in not a planar embedding and require_planar_embedding=True
+        """
+
         self.x = np.array(x, dtype=np.double).ravel()
         self.y = np.array(y, dtype=np.double).ravel()
         if len(self.x) != len(self.y):
             raise ValueError("x and y must be same size")
         self.N = len(self.x)
         self.node1, self.node2 = np.array(node1, dtype=int).ravel(), np.array(node2, dtype=int).ravel()
-        self.assert_edges_correct_shape()
+        self._assert_edges_correct_shape()
         self.E = len(self.node1)
-        self.assert_edges_contain_existing_nodes()
-        self.assert_no_self_loop()
+        self._assert_edges_contain_existing_nodes()
+        self._assert_no_self_loop()
 
         self.edge_sorter = np.arange(self.edge_count())
         if not _edges_are_sorted:
             self._sort_edges(node1, node2)
 
-        self.assert_nonsimple()
+        self._assert_nonsimple()
         self.edge_v_array = self._assign_edge_v_array()
 
         # quantities are computed and stored only when a method needs them.
@@ -79,42 +98,49 @@ class EmbeddedGraph:
         self.boundary_face_indices = None
 
         if require_single_component:
-            self.assert_single_component()
+            self._assert_single_component()
         if require_planar_embedding:
-            self.assert_planar_embedding()
-
-    def _assign_edge_v_array(self):
-        self.edge_v_array = VarRowArray(np.diff(np.append(np.flatnonzero(np.roll(self.node1, 1) - self.node1), len(self.node1))))
-        return self.edge_v_array
-
-    def assert_edges_correct_shape(self):
-        if len(self.node1) != len(self.node2):
-            raise ValueError("node1 and node2 must be same size")
-
-    def assert_edges_contain_existing_nodes(self):
-        if np.any((self.node1 < 0) | (self.node1 >= self.N) | (self.node2 < 0) | (self.node2 >= self.N)):
-            raise NodeNotExistError("node1,2 values must be in range(N)")
-
-    def assert_single_component(self):
-        if self.get_num_components() != 1:
-            raise NotSingleComponentError()
-
-    def assert_planar_embedding(self):
-        if not self.is_planar_embedding():
-            raise NotPlanarEmbeddingError()
-
-    def assert_no_self_loop(self):
-        if np.any(self.node1 == self.node2):
-            raise SelfLoopError("no edge is allowed to have identical end nodes.")
-
-    def assert_nonsimple(self):
-        if np.any((self.node1[:-1] == self.node1[1:]) & (self.node2[:-1] == self.node2[1:])):
-            raise NonSimpleError("no duplicate edges allowed.")
+            self._assert_planar_embedding()
 
     def get_edges(self):
+        """
+        Return node endpoints of all edges.
+
+        Returns
+        -------
+        node1, node2: (E,) arrays
+            node endpoints of all edges.
+
+        Notes
+        -----
+        - Edges are sorted such that node1 < node2
+        - After that, edges are lexographically sorted first by node1, then by node2.
+        """
         return self.node1, self.node2
 
     def get_edge_ids(self, node1, node2):
+        """
+        Return ids of edges with given endpoint nodes
+
+        Parameters
+        ----------
+        node1, node2: arrays in range(N)
+            endpoints of edges
+
+        Returns
+        -------
+        edge_ids: array with same size as node1 in range(E)
+            ids of edges
+
+        Raises
+        ------
+        EdgeNotExistError
+            If a queried node-pair does not exist.
+
+        Notes
+        -----
+        The id of an edge equals the position in the output .get_edges()
+        """
         node1, node2 = np.array(node1, dtype=int).ravel(), np.array(node2, dtype=int).ravel()
         mask = node1 < node2
         node1, node2 = np.where(mask, node1, node2), np.where(mask, node2, node1)
@@ -132,30 +158,40 @@ class EmbeddedGraph:
         return starts
 
     def add_nodes(self, x, y):
-        # if in_place:
-        #     self.x = np.append(self.x, np.array(x, dtype=np.double).ravel())
-        #     self.y = np.append(self.y, np.array(y, dtype=np.double).ravel())
-        #     self.N = len(self.x)
+        """
+        Add nodes to graph.
+
+        Parameters
+        ----------
+        x, y: arrays
+            coordinates of the nodes to be added to the graph
+
+        Returns
+        -------
+        new_graph: EmbeddedGraph
+            new graph with nodes added to it.
+        """
         return EmbeddedGraph(np.append(self.x, np.array(x, dtype=np.double).ravel()), np.append(self.y, np.array(y, dtype=np.double).ravel()), self.node1, self.node2, require_single_component=False)
 
     def add_edges(self, node1, node2, require_single_component=False, require_planar_embedding=False):
-        # if in_place:
-        #     self._reset_precomputed_quantities()
-        #     self.node1 = np.append(self.node1, np.array(node1, dtype=int).ravel())
-        #     self.node2 = np.append(self.node2, np.array(node2, dtype=int).ravel())
-        #     self.E = len(self.node1)
-        #     self._sort_edges(self.node1, self.node2)
-        #     self.assert_edges_correct_shape()
-        #     self.assert_edges_contain_existing_nodes()
-        #     self.assert_no_self_loop()
-        #     self.assert_nonsimple()
-        #     self._assign_edge_v_array()
-        #     if require_single_component:
-        #         self.assert_single_component()
-        #     if require_planar_embedding:
-        #         self.assert_planar_embedding()
-        #     return self
-        # else:
+        """
+        Add edges to graph.
+
+        Parameters
+        ----------
+        node1, node2: arrays in range(N)
+            endpoints of edges
+        require_single_component=False:
+            If True; raises error if the resulting graph is not single-component
+        require_planar_embedding=False:
+            If True; raises error if the resulting graph is not a planar embedding
+
+        Returns
+        -------
+        new_graph: EmbeddedGraph
+            new graph with edges added to it.
+
+        """
         return EmbeddedGraph(self.x, self.y, np.append(self.node1, np.array(node1, dtype=int).ravel()),
                              np.append(self.node2, np.array(node2, dtype=int).ravel()),
                              require_single_component=require_single_component,
@@ -163,26 +199,26 @@ class EmbeddedGraph:
 
     def add_nodes_and_edges(self, x, y, node1, node2, require_single_component=False,
                             require_planar_embedding=False):
-        # if in_place:
-        #     self._reset_precomputed_quantities()
-        #     self.x = np.append(self.x, np.array(x, dtype=np.double).ravel())
-        #     self.y = np.append(self.y, np.array(y, dtype=np.double).ravel())
-        #     self.N = len(self.x)
-        #     self.node1 = np.append(self.node1, np.array(node1, dtype=int).ravel())
-        #     self.node2 = np.append(self.node2, np.array(node2, dtype=int).ravel())
-        #     self.E = len(self.node1)
-        #     self._sort_edges(self.node1, self.node2)
-        #     self.assert_edges_correct_shape()
-        #     self.assert_edges_contain_existing_nodes()
-        #     self.assert_no_self_loop()
-        #     self.assert_nonsimple()
-        #     self._assign_edge_v_array()
-        #     if require_single_component:
-        #         self.assert_single_component()
-        #     if require_planar_embedding:
-        #         self.assert_planar_embedding()
-        #     return self
-        # else:
+        """
+        Add nodes and edges to graph.
+
+        Parameters
+        ----------
+        x, y: arrays
+            coordinates of the nodes to be added to the graph
+        node1, node2: arrays in range(N + x.size)
+            endpoints of edges. The i-th new node must be referred to by index N + i.
+        require_single_component=False:
+            If True; raises error if the resulting graph is not single-component
+        require_planar_embedding=False:
+            If True; raises error if the resulting graph is not a planar embedding
+
+        Returns
+        -------
+        new_graph: EmbeddedGraph
+            new graph with nodes and edges added to it.
+
+        """
         return EmbeddedGraph(np.append(self.x, np.array(x, dtype=np.double).ravel()),
                              np.append(self.y, np.array(y, dtype=np.double).ravel()),
                              np.append(self.node1, np.array(node1, dtype=int).ravel()),
@@ -191,6 +227,23 @@ class EmbeddedGraph:
                              require_planar_embedding=require_planar_embedding)
 
     def remove_nodes(self, node_ids, require_single_component=False, require_planar_embedding=False):
+        """
+        Remove nodes from graph.
+
+        Parameters
+        ----------
+        node_ids: int array in range(N)
+            ids of nodes to be removed
+        require_single_component=False:
+            If True; raises error if the resulting graph is not single-component
+        require_planar_embedding=False:
+            If True; raises error if the resulting graph is not a planar embedding
+
+        Returns
+        -------
+        new_graph: EmbeddedGraph
+            new graph with nodes removed from it.
+        """
         node_ids = np.array(node_ids, dtype=int).ravel()
         node_map = np.ones(self.node_count(), dtype=int)
         node_map[node_ids] = 0
@@ -198,101 +251,110 @@ class EmbeddedGraph:
         node_map[node_ids] = -1
         n1, n2 = node_map[self.node1], node_map[self.node2]
         edge_ids = (n1 >= 0) & (n2 >= 0)
-        # if in_place:
-        #     self._reset_precomputed_quantities()
-        #
-        #     edge_map = np.ones(self.edge_count(), dtype=int)
-        #     edge_map[edge_ids] = 0
-        #     edge_map = np.cumsum(edge_map) - edge_map
-        #
-        #     self.x = np.delete(self.x, node_ids)
-        #     self.y = np.delete(self.y, node_ids)
-        #     self.N = len(self.x)
-        #     self.node1 = n1[edge_ids]
-        #     self.node2 = n2[edge_ids]
-        #     self.edge_sorter = edge_map[np.delete(self.edge_sorter, np.argsort(self.edge_sorter)[edge_ids])]
-        #     self.E = len(self.node1)
-        #
-        #     self._assign_edge_v_array()
-        #     if require_single_component:
-        #         self.assert_single_component()
-        #     if require_planar_embedding:
-        #         self.assert_planar_embedding()
-        #     return self
-        # else:
         return EmbeddedGraph(np.delete(self.x, node_ids), np.delete(self.y, node_ids),
                              n1[edge_ids], n2[edge_ids], require_single_component=require_single_component,
                              require_planar_embedding=require_planar_embedding, _edges_are_sorted=True)
 
     def remove_edges_by_ids(self, edge_ids, require_single_component=False, require_planar_embedding=False):
-        # if in_place:
-        #     self._reset_precomputed_quantities()
-        #
-        #     edge_map = np.ones(self.edge_count(), dtype=int)
-        #     edge_map[edge_ids] = 0
-        #     edge_map = np.cumsum(edge_map) - edge_map
-        #
-        #     self.node1, self.node2 = np.delete(self.node1, edge_ids), np.delete(self.node2, edge_ids)
-        #     self.edge_sorter = edge_map[np.delete(self.edge_sorter, np.argsort(self.edge_sorter)[edge_ids])]
-        #
-        #     self.E = len(self.node1)
-        #     self._sort_edges(self.node1, self.node2)
-        #     self._assign_edge_v_array()
-        #     if require_single_component:
-        #         self.assert_single_component()
-        #     if require_planar_embedding:
-        #         self.assert_planar_embedding()
-        #     return self
-        # else:
+        """
+        Remove edges from graph with edge ids as input.
+
+        Parameters
+        ----------
+        edge_ids: int array in range(E)
+            ids of nodes to be removed
+        require_single_component=False:
+            If True; raises error if the resulting graph is not single-component
+        require_planar_embedding=False:
+            If True; raises error if the resulting graph is not a planar embedding
+
+        Returns
+        -------
+        new_graph: EmbeddedGraph
+            new graph with edges removed from it.
+        """
         return EmbeddedGraph(self.x, self.y, np.delete(self.node1, edge_ids),
                              np.delete(self.node2, edge_ids), _edges_are_sorted=True,
                              require_single_component=require_single_component,
                              require_planar_embedding=require_planar_embedding)
 
     def remove_edges(self, node1, node2, require_single_component=False, require_planar_embedding=False):
+        """
+        Remove edges from graph with node endpoints as input.
+
+        Parameters
+        ----------
+        node1, node2: int arrays in range(N)
+            ids of node endpoints of edges to be removed
+        require_single_component=False:
+            If True; raises error if the resulting graph is not single-component
+        require_planar_embedding=False:
+            If True; raises error if the resulting graph is not a planar embedding
+
+        Returns
+        -------
+        new_graph: EmbeddedGraph
+            new graph with edges removed from it.
+
+        Raises
+        ------
+        EdgeNotExistError
+            If a queried node-pair does not exist.
+        """
         return self.remove_edges_by_ids(self.get_edge_ids(node1, node2),
                                         require_single_component=require_single_component,
                                         require_planar_embedding=require_planar_embedding)
 
-    def _reset_precomputed_quantities(self):
-        self.F = None
-        self.faces_v_array = None
-        self.face_edges = None
-        self.face_nodes = None
-        self.face_lengths = None
-        self.determinant = None
-        self.areas = None
-        self.roll_ids = None
-        self.boundary_face_indices = None
-
     def coo(self):
+        """
+        Returns coordinates of nodes
+
+        Returns
+        -------
+        x, y: (N,) arrays
+            coordinates of nodes in graph
+        """
         return self.x, self.y
 
     def node_count(self):
+        """
+        Returns number of nodes in graph (abbreviated by N)
+        """
         return self.N
 
     def edge_count(self):
+        """
+        Returns number of edges in graph (abbreviated by E)
+        """
         return self.E
 
-    def get_faces(self, include_boundary_faces=True, to_list=False):
-        if self.face_edges is None:
-            self.face_edges, self.face_lengths = self._construct_faces()
-            self.faces_v_array = VarRowArray(self.face_lengths)
-            self.F = len(self.face_lengths)
-        if include_boundary_faces:
-            edges, lengths = self.face_edges, self.face_lengths
-        else:
-            if self.boundary_face_indices is None:
-                self.get_areas()
-            edges = self.faces_v_array.delete_rows(self.face_edges, self.boundary_face_indices)
-            lengths = np.delete(self.face_lengths, self.boundary_face_indices)
-        if to_list:
-            return VarRowArray(lengths).to_list(edges)
-        return edges, lengths
 
     def get_face_nodes(self, include_boundary_faces=True, to_list=False):
+        """
+        Returns all faces; a face is a list of all nodes in its boundary.
+
+        Two types of faces exist:
+         * boundary-faces: surround whole components. Nodes in clockwise order.
+         * internal-faces: surround individual faces. Nodes in counter-clockwise order.
+
+        Parameters
+        ----------
+        include_boundary_faces=True:
+            If True, also returns boundary faces in output. Otherwise only internal faces.
+        to_list=False:
+            If true, output is in form of list-of-lists, otherwise concatenated array.
+
+        Returns
+        -------
+        nodes:  list-of-lists or array
+            For all faces the nodes in its boundary. Either as list of lists or concatenated
+            in a single array.
+        lengths: (F,) int array
+            Array containing the number of nodes in each face
+
+        """
         if self.face_edges is None:
-            self.get_faces()
+            self._get_faces()
         if self.face_nodes is None:
             e_idx = self.face_edges % self.edge_count()
             self.face_nodes = np.where(self.face_edges < self.edge_count(), self.node1[e_idx], self.node2[e_idx])
@@ -308,19 +370,58 @@ class EmbeddedGraph:
         return nodes, lengths
 
     def face_count(self, include_boundary_faces=True):
+        """
+        Returns number of faces in graph (abbreviated by F)
+
+        Two types of faces exist:
+         * boundary-faces: surround whole components.
+         * internal-faces: surround individual faces.
+
+        Parameters
+        ----------
+        include_boundary_faces=True:
+            If True, also includes boundary faces in count. Otherwise only internal faces.
+
+        Returns
+        -------
+        face_count:
+            Returns number of faces
+        """
         if self.F is None:
-            self.get_faces()
+            self._get_faces()
         if include_boundary_faces:
             return self.F
         else:
             return self.F - len(self.get_boundary_faces())
 
     def is_planar_embedding(self):
+        """
+        Returns if graph is planar embedding, which is true if edges only intersect at their endpoints.
+        """
         if self.face_edges is None:
-            self.get_faces()
+            self._get_faces()
         return self.node_count() + self.face_count(False) == self.edge_count() + 1
 
     def get_areas(self, include_boundary_faces=True):
+        """
+        Returns signed area of faces in graph
+
+        Two types of faces exist:
+         * boundary-faces: surround whole components. Nodes in clockwise order,
+           so negative area.
+         * internal-faces: surround individual faces. Nodes in counter-clockwise
+           order, so positive area.
+
+        Parameters
+        ----------
+        include_boundary_faces=True:
+            If True, also includes boundary faces in output. Otherwise only internal faces.
+
+        Returns
+        -------
+        areas:  (F,) array
+            Returns areas of faces
+        """
         if self.areas is None:
             if self.determinant is None:
                 self._compute_determinant()
@@ -333,6 +434,23 @@ class EmbeddedGraph:
             return np.delete(self.areas, self.boundary_face_indices)
 
     def get_centroids(self, include_boundary_faces=True):
+        """
+        Returns centroids of faces in graph
+
+        Two types of faces exist:
+         * boundary-faces: surround whole components.
+         * internal-faces: surround individual faces.
+
+        Parameters
+        ----------
+        include_boundary_faces=True:
+            If True, also includes boundary faces in output. Otherwise only internal faces.
+
+        Returns
+        -------
+        x, y:  (F,) arrays
+            Returns coordinates of centroids of faces
+        """
         six_times_area = 6.0 * self.get_areas()
         mask = np.isclose(six_times_area, 0.0)
         six_times_area[mask] = 2.0 * self.face_lengths[mask]
@@ -363,7 +481,7 @@ class EmbeddedGraph:
         return self.boundary_face_indices
 
     def cycle_space(self, include_boundary_faces=False):
-        face_edges, face_lengths = self.get_faces(include_boundary_faces=include_boundary_faces)
+        face_edges, face_lengths = self._get_faces(include_boundary_faces=include_boundary_faces)
         E, F = self.edge_count(), len(face_lengths)
         indptr = np.append([0], np.cumsum(face_lengths))
         indices, data = face_edges % E, 1 - 2 * (face_edges // E)
@@ -390,7 +508,7 @@ class EmbeddedGraph:
         # optionally return orientation; True if face1 passes edge counterclockwise
         # first encountering its node with lowest index
         if self.face_edges is None:
-            self.get_faces()
+            self._get_faces()
         f = self.faces_v_array.rows()[np.argsort(self.face_edges)].astype(np.int64)
         E = self.edge_count()
         f1, f2 = f[:E], f[E:2 * E]
@@ -427,7 +545,7 @@ class EmbeddedGraph:
     def permute_nodes(self, permutation):
         # first precompute faces, because face order will be different if computed after node permutation.
         if self.face_edges is None:
-            self.get_faces()
+            self._get_faces()
         if self.face_nodes is None:
             self.get_face_nodes()
         if self.areas is None:
@@ -452,7 +570,7 @@ class EmbeddedGraph:
             raise ValueError("invalid permutation")
         inv_permutation = np.argsort(permutation)
         if self.face_edges is None:
-            self.get_faces()
+            self._get_faces()
         self.face_lengths = self.face_lengths[permutation]
         self.face_edges = self.faces_v_array.permute_rows(permutation, self.face_edges)
         if self.areas is not None:
@@ -500,6 +618,61 @@ class EmbeddedGraph:
                         nr += 1
         return self.fig, self.ax
 
+    def _assert_edges_correct_shape(self):
+        if len(self.node1) != len(self.node2):
+            raise ValueError("node1 and node2 must be same size")
+
+    def _assert_edges_contain_existing_nodes(self):
+        if np.any((self.node1 < 0) | (self.node1 >= self.N) | (self.node2 < 0) | (self.node2 >= self.N)):
+            raise NodeNotExistError("node1,2 values must be in range(N)")
+
+    def _assert_single_component(self):
+        if self.get_num_components() != 1:
+            raise NotSingleComponentError()
+
+    def _assert_planar_embedding(self):
+        if not self.is_planar_embedding():
+            raise NotPlanarEmbeddingError()
+
+    def _assert_no_self_loop(self):
+        if np.any(self.node1 == self.node2):
+            raise SelfLoopError("no edge is allowed to have identical end nodes.")
+
+    def _assert_nonsimple(self):
+        if np.any((self.node1[:-1] == self.node1[1:]) & (self.node2[:-1] == self.node2[1:])):
+            raise NonSimpleError("no duplicate edges allowed.")
+
+    def _assign_edge_v_array(self):
+        self.edge_v_array = VarRowArray(np.diff(np.append(np.flatnonzero(np.roll(self.node1, 1) - self.node1), len(self.node1))))
+        return self.edge_v_array
+
+    def _reset_precomputed_quantities(self):
+        self.F = None
+        self.faces_v_array = None
+        self.face_edges = None
+        self.face_nodes = None
+        self.face_lengths = None
+        self.determinant = None
+        self.areas = None
+        self.roll_ids = None
+        self.boundary_face_indices = None
+
+    def _get_faces(self, include_boundary_faces=True, to_list=False):
+        if self.face_edges is None:
+            self.face_edges, self.face_lengths = self._construct_faces()
+            self.faces_v_array = VarRowArray(self.face_lengths)
+            self.F = len(self.face_lengths)
+        if include_boundary_faces:
+            edges, lengths = self.face_edges, self.face_lengths
+        else:
+            if self.boundary_face_indices is None:
+                self.get_areas()
+            edges = self.faces_v_array.delete_rows(self.face_edges, self.boundary_face_indices)
+            lengths = np.delete(self.face_lengths, self.boundary_face_indices)
+        if to_list:
+            return VarRowArray(lengths).to_list(edges)
+        return edges, lengths
+
     def _sort_edges(self, node1, node2):
         node1, node2 = np.array(node1, dtype=int),  np.array(node2, dtype=int)
         mask = node1 < node2
@@ -515,7 +688,7 @@ class EmbeddedGraph:
 
     def _compute_roll_ids(self):
         if self.face_edges is None:
-            self.get_faces()
+            self._get_faces()
         if self.face_nodes is None:
             self.get_face_nodes()
         self.roll_ids = self.face_nodes[self.faces_v_array.roll(1)]
